#ifndef QXREDIS_LEXER_HPP_
#define QXREDIS_LEXER_HPP_

#include <QIODevice>
#include <QObject>

namespace qxredis
{

class Lexer : public QObject
{
	Q_OBJECT

public:

	Lexer( QIODevice*, QObject* = nullptr );
	virtual ~Lexer() = default;

	Q_SIGNAL void character( char );
	Q_SIGNAL void unsafeString( const QString& );
	Q_SIGNAL void safeString( const QByteArray& );

private:

	Q_SLOT void readData();

private:

	bool readCharacter();
	bool readLength();
	bool readUnsafeString();
	bool readSafeString();

	QIODevice* m_device;
	QByteArray m_buffer;

	enum
	{
		DoingNothing,
		ReadingLength,
		ReadingUnsafeString,
		ReadingSafeString
	} m_state;

	int m_crlf;
	int m_length;
}; // class Lexer

} // namespace QRedis

#endif // QREDIS_LEXER_H
