#ifndef QREDIS_REQUEST_H
#define QREDIS_REQUEST_H

#include <QObject>
#include <QScopedPointer>

#include <config.hpp>
#include <reply.hpp>

namespace qxredis
{

class QXREDIS_EXPORT RequestPrivate;

/**
 * @brief Represents a Redis command and its response
 */
class QXREDIS_EXPORT Request : public QObject
{
	Q_OBJECT

public:

	/**
	 * @brief Initializes the request
	 * @param parent the parent QObject
	 */
	explicit Request( QObject* parent = nullptr );

	/**
	 * @brief Destroys the request
	 */
	virtual ~Request() = default;

	/**
	 * @brief Waits for the reply to be received
	 * @param msecs the amount of time in milliseconds to wait
	 * @return true if the reply was received
	 */
	bool waitForReply( int msecs = 10000 );

	/**
	 * @brief Emitted when a reply is received
	 * @param reply the reply received
	 */
	Q_SIGNAL void reply( const qxredis::Reply& reply );

private:

	const QScopedPointer< RequestPrivate > d;
}; // class QXREDIS_EXPORT

} // namespace QRedis

#endif // QREDIS_REQUEST_H
